const axios = require('axios')

/**
 * triggerQualtricsWorkflow triggers a Qualtrics Workflow, sending the eventData JSON object to Qualtrics.
 * @param {string} qualtricsWorkflowURL
 * @param {string} qualtricsAPIToken
 * @param {Object} eventData
 * @return {Promise<void>}
 */
async function triggerQualtricsWorkflow(qualtricsWorkflowURL, qualtricsAPIToken, eventData) {

  await axios.post(qualtricsWorkflowURL, eventData, {
    headers: {
      'Content-Type': 'application/json',
      'X-API-TOKEN': qualtricsAPIToken
    }
  })
}

/**
 * getQualtricsAPIToken fetches the API token from secrets manager.
 * Code derived from AWS Secrets Manager code samples.
 * @param {string} region
 * @param {string} secretARN
 * @return {string}
 */
async function getQualtricsAPIToken(region, secretKeyName, secretARN) {
  var qualtricsAPIToken;

  // If you need more information about configurations or implementing the sample code, visit the AWS docs:
  // https://aws.amazon.com/developers/getting-started/nodejs/

  // Load the AWS SDK
  var AWS = require('aws-sdk'),
      region = region,
      secretName = secretARN,
      secret,
      decodedBinarySecret;

  // Create a Secrets Manager client
  var client = new AWS.SecretsManager({
      region: region
  });

  // In this sample we only handle the specific exceptions for the 'GetSecretValue' API.
  // See https://docs.aws.amazon.com/secretsmanager/latest/apireference/API_GetSecretValue.html
  // We rethrow the exception by default.

  var result = await client.getSecretValue({SecretId: secretName}).promise().catch(err => {
    if (err.code === 'DecryptionFailureException')
        // Secrets Manager can't decrypt the protected secret text using the provided KMS key.
        // Deal with the exception here, and/or rethrow at your discretion.
        throw err;
    else if (err.code === 'InternalServiceErrorException')
        // An error occurred on the server side.
        // Deal with the exception here, and/or rethrow at your discretion.
        throw err;
    else if (err.code === 'InvalidParameterException')
        // You provided an invalid value for a parameter.
        // Deal with the exception here, and/or rethrow at your discretion.
        throw err;
    else if (err.code === 'InvalidRequestException')
        // You provided a parameter value that is not valid for the current state of the resource.
        // Deal with the exception here, and/or rethrow at your discretion.
        throw err;
    else if (err.code === 'ResourceNotFoundException')
        // We can't find the resource that you asked for.
        // Deal with the exception here, and/or rethrow at your discretion.
        throw err;
    else
        throw err;
  })

  if ('SecretString' in result) {
      qualtricsAPIToken =  JSON.parse(result.SecretString)[secretKeyName];
  } else {
      let buff = new Buffer(result.SecretBinary, 'base64');
      qualtricsAPIToken = buff.toString('ascii')[secretKeyName];
  }

  if (!qualtricsAPIToken) {
    throw new Error("Token not found");
  }

  return qualtricsAPIToken;
}


// --------------- Main handler -----------------------
exports.handler = async (event, context, callback) => {
  const parameters = event.Details.Parameters

  // qualtricsEventURL is a required parameters.
  // All other parameters included are passed to the Qualtrics JSON Event.
  const {
    qualtricsEventURL
  } = parameters

  var region = process.env.AWS_REGION;
  var secretKeyName = process.env.QUALTRICS_API_TOKEN_KEY;
  var secretARN = process.env.QUALTRICS_API_TOKEN_ARN;

  var qualtricsAPIToken = await getQualtricsAPIToken(region, secretKeyName, secretARN);
  await triggerQualtricsWorkflow(qualtricsEventURL, qualtricsAPIToken, parameters);

  callback(null, {});
}
